using System;
using System.Collections;
using System.Collections.Generic;

namespace Dashboard.Controls
{
    public enum ClassChangeType
    {
        Added,
        Removed,
    }

    public record struct ClassChanged(object? Owner, string ClassName, ClassChangeType Type);

    public class ClassSet : ICollection<string>
    {
        public int Count => _classes.Count;
        public bool IsReadOnly => false;
        public object? Owner { get; }
        public event EventHandler<ClassChanged>? ClassChanged;

        private readonly HashSet<string> _classes = new HashSet<string>();

        public ClassSet(object? owner)
        {
            Owner = owner;
        }

        public IEnumerator<string> GetEnumerator() => _classes.GetEnumerator();

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public void Add(string item)
        {
            if (_classes.Add(item))
            {
                OnClassAdded(item);
            }
        }

        public void Clear()
        {
            foreach (string @class in _classes)
                OnClassRemoved(@class);

            _classes.Clear();
        }

        public bool Contains(string item) => _classes.Contains(item);

        public void CopyTo(string[] array, int arrayIndex) => _classes.CopyTo(array, arrayIndex);

        public bool Remove(string item)
        {
            if (_classes.Remove(item))
            {
                OnClassRemoved(item);
                return true;
            }

            return false;
        }

        private void OnClassAdded(string @class)
        {
            ClassChanged?.Invoke(this, new ClassChanged(Owner, @class, ClassChangeType.Added));
        }

        private void OnClassRemoved(string @class)
        {
            ClassChanged?.Invoke(this, new ClassChanged(Owner, @class, ClassChangeType.Removed));
        }
    }
}
