using System;
using Dashboard.Windowing;

namespace Dashboard
{
    public interface IWindowFactory
    {
        /// <summary>
        /// Creates a window. It could be a virtual window, or a physical window.
        /// </summary>
        /// <returns>A window.</returns>
        IWindow CreateWindow();

        /// <summary>
        /// Always creates a physical window.
        /// </summary>
        /// <returns>A physical window.</returns>
        IPhysicalWindow CreatePhysicalWindow();
    }

    public interface IDashboardBackend : IDisposable, IWindowFactory
    {
        void Initialize();

        void RunEvents(bool wait);

        void Leave();
    }
}